c
c ---------------------------------------------------------------
c
        subroutine filpatch2(level,nvar,valbig,aux,naux,
     1                      time,mitot,mjtot,
     2                      nrowst,ncolst,ilo,ihi,jlo,jhi)

c :::::::::::::::::::::::::::: FILPATCH :::::::::::::::::::::::::;
c
c  fill the portion of valbig from rows  nrowst
c                             and  cols  ncolst
c  the patch can also be described by the corners (xlp,ybp) by (xrp,ytp).
c  vals are needed at time time, and level level,
c
c  first fill with  values obtainable from the level level
c  grids. if any left unfilled, then enlarge remaining rectangle of
c  unfilled values by 1 (for later linear interp), and recusively
c  obtain the remaining values from  coarser levels.
c
c :::::::::::::::::::::::::::::::::::::::;:::::::::::::::::::::::;

      implicit double precision (a-h,o-z)

      include  "call.i"

      logical   set, sticksout
      dimension valbig(mitot,mjtot,nvar), aux(mitot,mjtot,naux)

      dimension coarseval(9,3)
      dimension fineval(3)
      dimension auxcoarse(9)

      logical velflag(mitot,mjtot)
      logical dryflag(mitot,mjtot)


      common /tolerance/ depthtolerance


      iadflag(i,j)    =  locuse + i-1+(j-1)*nrowp
      ivalc(i,j,ivar) =  loccrse + (i - 1) + nrowc*(j - 1)
     &                     + nrowc*ncolc*(ivar-1)
c
c     # index into first component of aux = bathymetry:
      iauxc(i,j) =  locauxc + (i - 1) + nrowc*(j - 1)

      sticksout(iplo,iphi,jplo,jphi)  =
     &            (iplo .lt. 0 .or. jplo .lt. 0 .or.
     &             iphi .ge. iregsz(levc) .or. jphi .ge. jregsz(levc))
c
c We begin by filling values for grids at level level. If all values can be
c filled in this way, we return;

        hxf     = hxposs(level)
        hyf     = hyposs(level)
        xlp     = xlower + ilo*hxf
        xrp     = xlower + (ihi+1)*hxf
        ybp     = ylower + jlo*hyf
        ytp     = ylower + (jhi+1)*hyf
        nrowp   = ihi - ilo + 1
        ncolp   = jhi - jlo + 1
        locuse  = igetsp(nrowp*ncolp)

c 2/27/02 : added naux to intfil call.
        call intfil
     &  (valbig,mitot,mjtot,time,locuse,nrowst,ncolst,
     &   ilo,ihi,jlo,jhi,level,nvar,naux)
c
c Trimbd returns set = true if all of the entries are filled (=1.).
c set = false, otherwise. If set = true, then no other levels are
c are required to interpolate, and we return.
c
c Note that the used array is filled entirely in intfil, i.e. the
c marking done there also takes into account the points filled by
c the boundary conditions. bc2amr will be called later (from bound), after
c all 4 boundary pieces filled.

        call trimbd(alloc(locuse),nrowp,ncolp,set,il,ir,jb,jt)

        if (set) then
           call bc2amr(valbig,aux,mitot,mjtot,nvar,naux,
     1                 hxf,hyf,level,time,
     2                 xlp,xrp,ybp,ytp,
     3                 xlower,ylower,xupper,yupper,
     4                 xperdom,yperdom)
           call reclam(locuse,nrowp*ncolp)
           return
        else if (level .eq. 1) then
           write(outunit,*)" error in filpatch - level 1 not set"
           write(outunit,900) nrowst,ncolst
           write(*,*)" error in filpatch - level 1 not set"
           write(*,900) nrowst,ncolst
900        format("starting at row: ",i4," col ",i4)
           stop
        endif

c set = false. we will have to interpolate some values from coarser
c levels. We begin by initializing the level level arrays, so that we can use
c recursive formulation for interpolating.
c IS THIS TRUE ? - the fine grid patch remaining unfilled is always
c anchored to a coarse cell.

        levc = level - 1
        hxc  = hxposs(levc)
        hyc  = hyposs(levc)

        isl  = il + ilo - 1
        isr  = ir + ilo - 1
        jsb  = jb + jlo - 1
        jst  = jt + jlo - 1
c
c       coarsen
        lratiox = intratx(levc)
        lratioy = intraty(levc)
        iplo   = (isl-lratiox+nghost*lratiox)/lratiox - nghost
        jplo   = (jsb-lratioy+nghost*lratioy)/lratioy - nghost
        iphi   = (isr+lratiox)/lratiox
        jphi   = (jst+lratioy)/lratioy


        xlc  =  xlower + iplo*hxc
        ybc  =  ylower + jplo*hyc
        xrc  =  xlower + (iphi+1)*hxc
        ytc  =  ylower + (jphi+1)*hyc

        nrowc   =  iphi - iplo + 1
        ncolc   =  jphi - jplo + 1
        ntot    = nrowc*ncolc*(nvar+naux)
        loccrse = igetsp(ntot)
        locauxc = loccrse + nrowc*ncolc*nvar
        if (naux.gt.0) then
              maxmx = nrowc - 2*nghost
              mx = maxmx
              maxmy = ncolc - 2*nghost
              my = maxmy
              xl = xlc + nghost*hxc
              yb = ybc + nghost*hyc
              call setaux(maxmx,maxmy,nghost,mx,my,xl,yb,hxc,hyc,
     &                    naux,alloc(locauxc))
        endif


        if ((xperdom .or. yperdom) .and.
     &       sticksout(iplo,iphi,jplo,jphi)) then
             call prefil3(levc,nvar,alloc(loccrse),alloc(locauxc),naux,
     1                    time,nrowc,ncolc,1,1,
     2                    iplo,iphi,jplo,jphi)
        else
          call filpatch3(levc,nvar,alloc(loccrse),alloc(locauxc),naux,
     1                   time,nrowc,ncolc,1,1,
     2                   iplo,iphi,jplo,jphi)
        endif


c       interpolate back up

20      continue


c	#set velflag flags to false
	do ic=1,mitot
	do jc=1,mjtot
             dryflag(ic,jc)=.false.
	     velflag(ic,jc)=.false.
	enddo
	enddo
        toldry=(depthtolerance)
c	#loop through all fine cells interpolating eta,hu,hv

        do 100 iff = 1,nrowp
          ic = 2 + (iff - (isl - ilo) - 1)/lratiox
          eta1 = (-0.5d0+dble(mod(iff-1,lratiox)))/dble(lratiox)

        do 100 jf  = 1,ncolp
          jc = 2 + (jf -(jsb-jlo)-1)/lratioy
          eta2 = (-0.5d0+dble(mod(jf -1,lratioy)))/dble(lratioy)

           flag = alloc(iadflag(iff,jf))
           if (flag .eq. 0.0) then


	    if (jc.gt.mjtot.or.ic.gt.mitot) then
		write(*,*) 'FILPATCH: WARNING mitot<ic'
	    endif

c	   # coarse values surrounding the current coarse value are set
c.	   # These 9 cells are ordered as follows:
c	   # left to right, and bottom to top
c	   # This makes ic,jc the number 5 cell

                auxcoarse(1) = alloc(iauxc(ic-1,jc-1))
                auxcoarse(2) = alloc(iauxc(ic  ,jc-1))
                auxcoarse(3) = alloc(iauxc(ic+1,jc-1))
                auxcoarse(4) = alloc(iauxc(ic-1,jc  ))
                auxcoarse(5) = alloc(iauxc(ic  ,jc  ))
                auxcoarse(6) = alloc(iauxc(ic+1,jc  ))
                auxcoarse(7) = alloc(iauxc(ic-1,jc+1))
                auxcoarse(8) = alloc(iauxc(ic  ,jc+1))
                auxcoarse(9) = alloc(iauxc(ic+1,jc+1))

                do ivar=1,nvar
                  coarseval(1,ivar) = alloc(ivalc(ic-1,jc-1,ivar))
                  coarseval(2,ivar) = alloc(ivalc(ic  ,jc-1,ivar))
                  coarseval(3,ivar) = alloc(ivalc(ic+1,jc-1,ivar))
                  coarseval(4,ivar) = alloc(ivalc(ic-1,jc  ,ivar))
                  coarseval(5,ivar) = alloc(ivalc(ic  ,jc  ,ivar))
                  coarseval(6,ivar) = alloc(ivalc(ic+1,jc  ,ivar))
                  coarseval(7,ivar) = alloc(ivalc(ic-1,jc+1,ivar))
                  coarseval(8,ivar) = alloc(ivalc(ic  ,jc+1,ivar))
                  coarseval(9,ivar) = alloc(ivalc(ic+1,jc+1,ivar))
                enddo


c               # convert h to surface eta = h + bath
c	        # if h=0, make sure hu,hv = 0.

               do icirc=1,9
                  if (coarseval(icirc,1).le.toldry) then
                     coarseval(icirc,2)=0.d0
                     coarseval(icirc,3)=0.d0
                     coarseval(icirc,1)=coarseval(icirc,1)
     &				+auxcoarse(icirc)
                  else
                     coarseval(icirc,1)=coarseval(icirc,1)
     &                      	+auxcoarse(icirc)
                  endif
               enddo

c		# if hcoarse=0, then set eta= average of surrounding cells
	        if (coarseval(5,1)-auxcoarse(5).le.toldry) then
		 etaav=0.d0
		 coarseval(5,2)=0.d0
		 coarseval(5,3)=0.d0
		 wet=0.d0
		 do icirc=1,9
		  if (coarseval(icirc,1)-auxcoarse(icirc).gt.toldry) then
		    wet=wet+1.d0
		    etaav=etaav+ coarseval(icirc,1)
		  endif
		 enddo
		 if (wet.gt.0.d0) etaav=etaav/wet
		 coarseval(5,1)=min(coarseval(5,1),etaav)
	        endif


               do 101  ivar = 1,nvar

                   s1p=coarseval(6,ivar)-coarseval(5,ivar)
                   s1m=coarseval(5,ivar)-coarseval(4,ivar)

		   if (coarseval(6,1)-auxcoarse(6).gt.toldry
     &		      .and.coarseval(4,1)-auxcoarse(4).gt.toldry) then
                   	slopex=dmin1(dabs(s1p),dabs(s1m))*
     &                	dsign(1.d0,coarseval(6,ivar)-coarseval(4,ivar))
     		   elseif (coarseval(6,1)-auxcoarse(6).gt.toldry) then
		   	slopex=s1p
		   elseif (coarseval(4,1)-auxcoarse(4).gt.toldry) then
		   	slopex=s1m
		   else
		   	slopex=0.d0
		   endif

                   if (s1m*s1p.le.0.d0) then
                      slopex=0.d0
                   endif

                   s1p=coarseval(8,ivar)-coarseval(5,ivar)
                   s1m=coarseval(5,ivar)-coarseval(2,ivar)

		   if (coarseval(8,1)-auxcoarse(8).gt.toldry
     &		      .and.coarseval(2,1)-auxcoarse(2).gt.toldry) then
                   	slopey=dmin1(dabs(s1p),dabs(s1m))*
     &                	dsign(1.d0,coarseval(8,ivar)-coarseval(2,ivar))
     		   elseif (coarseval(8,1)-auxcoarse(8).gt.toldry) then
		        slopey=s1p
		   elseif (coarseval(2,1)-auxcoarse(2).gt.toldry) then
		        slopey=s1m
		   else
		        slopey=0.d0
		   endif


                   if (s1m*s1p.le.0.d0) then
                      slopey=0.d0
                   endif

c:::::::::::::::::::limited slope interpolation ::::::::::::
                   fineval(ivar)=coarseval(5,ivar) + eta1*slopex
     &                  + eta2*slopey


                   if (ivar.eq.1) then
                      fineval(1)=fineval(1)
     &                     -aux(iff+nrowst-1,jf+ncolst-1,1)
                      fineval(1)=max(fineval(1),0.d0)
                   else
                      h=coarseval(5,1)-auxcoarse(5)
		      velcoarse=0.d0
		      if (h.gt.toldry) velcoarse=coarseval(5,ivar)/h
                      velmax=velcoarse
                      velmin=velcoarse
		      do icirc=1,9
                         h=coarseval(icirc,1)-auxcoarse(icirc)
		         vel=0.d0
		         if (h.gt.toldry) vel=coarseval(icirc,ivar)/h
                         velmax=max(vel,velmax)
                         velmin=min(vel,velmin)
		      enddo

c		      #determine if velocities violate min,max vel.
		      if (fineval(1).gt.toldry) then
			velfine=fineval(ivar)/fineval(1)
		      else
                        velflag(ic,jc)=.true.
		      endif
		      if (velfine.gt.velmax.or.velfine.lt.velmin) then
     			velflag(ic,jc)=.true.
		      endif

		   endif
                   valbig(iff+nrowst-1,jf+ncolst-1,ivar) = fineval(ivar)
 101               continue

           endif

 100     continue

c	# repeat the whole process resetting if velflag=.true.

        do iff = 1,nrowp
          ic = 2 + (iff - (isl - ilo) - 1)/lratiox
          eta1 = (-0.5d0+dble(mod(iff-1,lratiox)))/dble(lratiox)

        do jf  = 1,ncolp
          jc = 2 + (jf -(jsb-jlo)-1)/lratioy
          eta2 = (-0.5d0+dble(mod(jf -1,lratioy)))/dble(lratioy)

           flag = alloc(iadflag(iff,jf))
           if (flag .eq. 0.0) then
	   if (velflag(ic,jc)) then

c	   # coarse values surrounding the current coarse value are set
c.	   # These 9 cells are ordered as follows:
c	   # left to right, and bottom to top
c	   # This makes ic,jc the number 5 cell

                auxcoarse(1) = alloc(iauxc(ic-1,jc-1))
                auxcoarse(2) = alloc(iauxc(ic  ,jc-1))
                auxcoarse(3) = alloc(iauxc(ic+1,jc-1))
                auxcoarse(4) = alloc(iauxc(ic-1,jc  ))
                auxcoarse(5) = alloc(iauxc(ic  ,jc  ))
                auxcoarse(6) = alloc(iauxc(ic+1,jc  ))
                auxcoarse(7) = alloc(iauxc(ic-1,jc+1))
                auxcoarse(8) = alloc(iauxc(ic  ,jc+1))
                auxcoarse(9) = alloc(iauxc(ic+1,jc+1))

                do ivar=1,nvar
                  coarseval(1,ivar) = alloc(ivalc(ic-1,jc-1,ivar))
                  coarseval(2,ivar) = alloc(ivalc(ic  ,jc-1,ivar))
                  coarseval(3,ivar) = alloc(ivalc(ic+1,jc-1,ivar))
                  coarseval(4,ivar) = alloc(ivalc(ic-1,jc  ,ivar))
                  coarseval(5,ivar) = alloc(ivalc(ic  ,jc  ,ivar))
                  coarseval(6,ivar) = alloc(ivalc(ic+1,jc  ,ivar))
                  coarseval(7,ivar) = alloc(ivalc(ic-1,jc+1,ivar))
                  coarseval(8,ivar) = alloc(ivalc(ic  ,jc+1,ivar))
                  coarseval(9,ivar) = alloc(ivalc(ic+1,jc+1,ivar))
                enddo

c               # convert h to surface eta = h + bath
c	        # if h=0, make sure hu,hv = 0.

               do icirc=1,9
                  if (coarseval(icirc,1).le.toldry) then
                     coarseval(icirc,2)=0.d0
                     coarseval(icirc,3)=0.d0
                     coarseval(icirc,1)=coarseval(icirc,1)
     &				+auxcoarse(icirc)
                  else
                     coarseval(icirc,1)=coarseval(icirc,1)
     &                      	+auxcoarse(icirc)
                  endif
               enddo

c		# if hcoarse=0, then set eta= average of surrounding cells
	        if (coarseval(5,1)-auxcoarse(5).le.toldry) then
		 etaav=0.d0
		 coarseval(5,2)=0.d0
		 coarseval(5,3)=0.d0
		 wet=0.d0
		 do icirc=1,9
		  if (coarseval(icirc,1)-auxcoarse(icirc).gt.toldry) then
		    wet=wet+1.d0
		    etaav=etaav + coarseval(icirc,1)
		  endif
		 enddo
		 if (wet.gt.0.d0) etaav=etaav/wet
		 coarseval(5,1)=min(coarseval(5,1),etaav)
	        endif

                do ivar = 1,nvar
                   s1p=coarseval(6,ivar)-coarseval(5,ivar)
                   s1m=coarseval(5,ivar)-coarseval(4,ivar)

		   if (coarseval(6,1)-auxcoarse(6).gt.toldry
     &		      .and.coarseval(4,1)-auxcoarse(4).gt.toldry) then
                   	slopex=dmin1(dabs(s1p),dabs(s1m))*
     &                	dsign(1.d0,coarseval(6,ivar)-coarseval(4,ivar))
     		   elseif (coarseval(6,1)-auxcoarse(6).gt.toldry) then
		   	slopex=s1p
		   elseif (coarseval(4,1)-auxcoarse(4).gt.toldry) then
		   	slopex=s1m
		   else
		   	slopex=0.d0
		   endif

                   if (s1m*s1p.le.0.d0) then
                      slopex=0.d0
                   endif

                   s1p=coarseval(8,ivar)-coarseval(5,ivar)
                   s1m=coarseval(5,ivar)-coarseval(2,ivar)

		   if (coarseval(8,1)-auxcoarse(8).gt.toldry
     &		      .and.coarseval(2,1)-auxcoarse(2).gt.toldry) then
                   	slopey=dmin1(dabs(s1p),dabs(s1m))*
     &                	dsign(1.d0,coarseval(8,ivar)-coarseval(2,ivar))
     		   elseif (coarseval(8,1)-auxcoarse(8).gt.toldry) then
		        slopey=s1p
		   elseif (coarseval(2,1)-auxcoarse(2).gt.toldry) then
		        slopey=s1m
		   else
		        slopey=0.d0
		   endif


                   if (s1m*s1p.le.0.d0) then
                      slopey=0.d0
                   endif


c:::::::::::::::::::limited slope interpolation ::::::::::::
                   fineval(ivar)=coarseval(5,ivar) + eta1*slopex
     &                  + eta2*slopey

                   if (ivar.eq.1) then
                      fineval(1)=fineval(1)
     &                     -aux(iff+nrowst-1,jf+ncolst-1,1)
                      fineval(1)=max(fineval(1),0.d0)
                   else
                      h=coarseval(5,1)-auxcoarse(5)
		      velcoarse=0.d0
		      if (h.gt.toldry) velcoarse=coarseval(5,ivar)/h
		      fineval(ivar)=fineval(1)*velcoarse
		   endif
                valbig(iff+nrowst-1,jf+ncolst-1,ivar) = fineval(ivar)
		enddo
	   endif
	   endif
	enddo
	enddo

        call bc2amr(valbig,aux,mitot,mjtot,nvar,naux,
     1              hxf,hyf,level,time,
     2              xlp,xrp,ybp,ytp,
     3              xlower,ylower,xupper,yupper,
     4              xperdom,yperdom)

        call reclam(loccrse,ntot)

        call reclam(locuse,nrowp*ncolp)

        return
        end
