
c======================================================================
      subroutine rpn2(ixy,maxm,meqn,mwaves,mbc,mx,ql,qr,auxl,auxr,
     &                  fwave,s,amdq,apdq)
c======================================================================


c Solves normal Riemann problems for the 2D SHALLOW WATER equations 
c     with topography:
c     #        h_t + (hu)_x + (hv)_y = 0                           #
c     #        (hu)_t + (hu^2 + 0.5gh^2)_x + (huv)_y = -ghb_x      #
c     #        (hv)_t + (huv)_x + (hv^2 + 0.5gh^2)_y = -ghb_y      #

c On input, ql contains the state vector at the left edge of each cell
c     qr contains the state vector at the right edge of each cell
c
c This data is along a slice in the x-direction if ixy=1 
c     or the y-direction if ixy=2.
       
c  Note that the i'th Riemann problem has left state qr(i-1,:)
c     and right state ql(i,:)
c  From the basic clawpack routines, this routine is called with
c     ql = qr
c
c======================================================================
c              last modified 10/24/05
c======================================================================

      implicit none

      integer i,ixy,mx,m,mw,mbc,meqn,mwaves,maxm,mu,nv,maxiter,mcapa

      double precision  fwave(1-mbc:maxm+mbc, meqn, mwaves)
      double precision  s(1-mbc:maxm+mbc, mwaves)
      double precision  ql(1-mbc:maxm+mbc, meqn)
      double precision  qr(1-mbc:maxm+mbc, meqn)
      double precision  apdq(1-mbc:maxm+mbc, meqn)
      double precision  amdq(1-mbc:maxm+mbc, meqn)
      double precision  auxl(1-mbc:maxm+mbc, *)
      double precision  auxr(1-mbc:maxm+mbc, *)

      double precision r(4,4)
      double precision lambda(4)
      double precision beta(4)
      double precision del(4)

      double precision abs_zero,abs_tol,grav,g,depthtolerance
      double precision hL,hR,uL,uR,huR,huL,hvR,hvL,vR,vL
      double precision dfdh,f1L,f2L,f1R,f2R,bR,bL
      double precision delq1,delq2,delf1,delf2,delphi,delnorm
      double precision hm,hum,um,u1m,u2m,s1m,s2m
      double precision uhat,chat,roe1,roe2,sR,sL
      double precision tw,Rearth,pi,R1,R2,dxdc

      logical solidwallL,solidwallR,rare1,rare2
      logical debug,singular
c=============lapack variables=======================================
      integer MN,INFO,LWORK,MROWS,NCOLMS,RANK,NRHS,LDA,LDB
      integer IPIV(4)

      double precision A(4,4)

      double precision RCOND
      double precision MINMN(4)
      double precision WORK(30)
c====================================================================

      common /comrp/ grav
      common /tolerance/ depthtolerance
      common /earth/ Rearth,R1,R2,pi
      common /latlongrid/ mcapa

      g=grav
      abs_tol=depthtolerance
      abs_zero=1.d-12

c======================================================================
c         loop through Riemann problems at each grid cell
c======================================================================
c     #reinitialize
      do i=1-mbc,mx+mbc
      	 do mw=1,mwaves
       	     s(i,mw)=0.d0
             do m=1,meqn
                fwave(i,m,mw)=0.d0
             enddo
         enddo
      enddo

c     #set normal direction
       if (ixy.eq.1) then
          mu=2
          nv=3
       else
          mu=3
          nv=2
       endif


      do 30 i = 2-mbc, mx+mbc

c     #reinitialize
	 do mw=1,4
	    lambda(mw)=0.d0
	    beta(mw)=0.d0
	    do m=1,4
	       r(m,mw)=0.d0
	    enddo
	 enddo



c      #notify if a bad riemann problem from the start
       	 if (qr(i-1,1).lt.0.d0 .or. ql(i,1).lt.0.d0) then
            if (qr(i-1,1).lt.-abs_tol.or. ql(i,1).lt.-abs_tol) then
	    	write(*,*) 'RPN2: Bad beginning value: i,hl,hr='
     &		,i,qr(i-1,1),ql(i,1)
     	    endif
            if (qr(i-1,1).lt.0.d0) then
              do m=1,meqn
                qr(i-1,m)=0.d0
                ql(i-1,m)=0.d0
              enddo
            endif
            if (ql(i,1).lt.0.d0) then
              do m=1,meqn
                qr(i,m)=0.d0
                ql(i,m)=0.d0
              enddo
            endif
         endif

         hL=qr(i-1,1)
         hR=ql(i,1)
         huL=qr(i-1,mu)
         huR=ql(i,mu)
         hvL=qr(i-1,nv)
         hvR=ql(i,nv)
         bL=auxr(i-1,1)
         bR=auxl(i,1)

	 if ((hL.le.abs_tol).and.(hR.le.abs_tol)) then
	    go to 30
	 endif

         if (hL.le.abs_tol) then
	    bL=hL+bL
            hL=0.d0
            huL=0.d0
            hvL=0.d0
	    uL=0.d0
	    vL=0.d0
	 else
	    uL=huL/hL
	    vL=hvL/hL
         endif

         if (hR.le.abs_tol) then
	    bR=hR+bR
            hR=0.d0
            huR=0.d0
            hvR=0.d0
	    uR=0.d0
	    vR=0.d0
	 else
	    uR=huR/hR
	    vR=hvR/hR
         endif
c=======================================================================
c  Initialize solid-wall problem for a wet/dry interface if necessary
c=======================================================================

	solidwallL=.false.
	solidwallR=.false.

	if (hL.le.abs_tol) then
	   if (bL.gt.bR+hR) then
		call riemann(hR,hR,-uR,uR,hm,s1m,s2m,rare1,rare2,10)
		if (bL.gt.bR+hm) then
                   solidwallL=.true.
		   hL=hR
		   uL=-uR
                   huL=-huR
		   bL=bR
		   vL=vR
		   hvL=hvR
		endif
	   endif
	endif

	if (hR.le.abs_tol) then
	   if (bR.gt.bL+hL) then
		call riemann(hL,hL,uL,-uL,hm,s1m,s2m,rare1,rare2,10)
                if (bR.gt.bL+hm) then
	   	   solidwallR=.true.
		   hR=hL
 		   huR=-huL
		   uR=-uL
		   bR=bL
		   vR=vL
		   hvR=hvL
		endif
	   endif
	endif

c=======================================================================
c	set-differences to be decomposed into eigenvectors
c=======================================================================
        f1L=huL
        f2L=0.5d0*g*hL**2 + hL*uL**2
        f1R=huR
        f2R=0.5d0*g*hR**2 + hR*uR**2

        delq1=hR-hL
        delq2=f1R-f1L
        delf1=f1R-f1L
        delf2=f2R-f2L
	delphi=bR-bL


	del(1)=delq1
	del(2)=delq2
	del(3)=delf2
	del(4)=delphi


c=====================================================================
c   Determine Riemann structure and set-up eigenvectors
c=====================================================================
	 maxiter=1
	 if ((.not.solidwallL).and.(.not.solidwallR)) then
	     call riemann(hL,hR,uL,uR,hm,s1m,s2m,rare1,rare2,maxiter)
	 endif

        sL=uL-sqrt(g*hL)
        sR=uR+sqrt(g*hR)

	if (hL.lt.abs_tol) then
	   sL=s2m
	endif
	if (hR.lt.abs_tol) then
	   sR=s1m
	endif

        uhat=sqrt(g*hL)*uL + sqrt(g*hR)*uR
        uhat=uhat/(sqrt(g*hR)+sqrt(g*hL))
        chat=sqrt(g*0.5d0*(hR+hL))
        roe1=uhat-chat
        roe2=uhat+chat

        lambda(1)=min(sL,roe1)
        lambda(3)=max(sR,roe2)
        lambda(2)=.5d0*(lambda(1)+lambda(3))
        lambda(4)=0.d0

	dfdh=max(-sL*sR,-roe1*roe2)
	dfdh=max(dfdh,-uL*uR+g*0.5d0*(hR+hL))

        do mw=1,4
	   r(1,mw)=1.d0
	   r(2,mw)=lambda(mw)
	   r(3,mw)=(lambda(mw))**2
	   r(4,mw)=0.d0
	enddo


	if (dabs(dfdh).gt.abs_zero) then
	   r(1,4)=-(g*0.5d0*(hR+hL))/(dfdh)
	   r(2,4)=0.d0
	   r(3,4)=-g*0.5d0*(hR+hL)
	   r(4,4)=1.d0
	else
c	 write(*,*) 'Supersonic Flow Transition'
	   r(1,4)=0.d0
	   r(2,4)=0.d0
	   r(3,4)=-g*.5d0*(hR+hL)
	   r(4,4)=1.d0
	endif

	do mw=1,4
	do m=1,4
	    A(m,mw)=r(m,mw)
	enddo
	enddo

	singular=.false.

	if (.not.singular) then
c	#params for lapack routines
	   MN=4
	   NRHS=1
	   LWORK=30
	   LDA=4
	   LDB=4
	   call DGESV(MN, NRHS, A, LDA, IPIV, del, LDB, INFO )
	   if (INFO.eq.0) then
		do mw=1,4
		   beta(mw)=del(mw)
		enddo
	   elseif (INFO.gt.0) then
		singular=.true.
	   endif
	endif
c===============determine if r is singular, and solve rectangular system====
	if (singular) then
	   write(*,*) 'RP1 ERROR: degenerate eigenspace'
	   write(*,*) 'RP1: finding least squares solution'
	     	NCOLMS=4
	     	MROWS=4
	     	NRHS=1
	     	LWORK=30
	     	LDA=4
	     	LDB=4
             	call DGELSS(MROWS,NCOLMS,NRHS,A,LDA,del,LDB,S,
     &                   RCOND,RANK,WORK,LWORK,INFO)
	   	if (INFO.eq.0) then
		do mw=1,4
		   beta(mw)=del(mw)
		enddo
	   	endif
	endif
c	=========================================================================
	if (.not.(INFO.eq.0)) then
	  write(*,*) 'RP1: ERROR SOLVING SYSTEM'
	  stop
	endif
c==================determine speeds and fwaves ======================
	 tw=1.d0
	 if (solidwallL) then
	    do mw=1,2
		beta(mw)=0.d0
		lambda(mw)=0.d0
	    enddo
	    tw=0.d0
	 elseif (solidwallR) then
  	    do mw=2,3
		beta(mw)=0.d0
		lambda(mw)=0.d0
	    enddo
	    tw=0.d0
	 endif

	 beta(4)=0.d0
         do mw=1,mwaves
	    s(i,mw)=lambda(mw)
            fwave(i,1,mw)=beta(mw)*r(2,mw)
	    fwave(i,mu,mw)=beta(mw)*r(3,mw)
         enddo
	    fwave(i,nv,1)=beta(1)*r(2,1)*vL
	    fwave(i,nv,3)=beta(3)*r(2,3)*vR
	    fwave(i,nv,2)=tw*(hR*uR*vR-hL*uL*vL)
     &         -fwave(i,nv,1)-fwave(i,nv,3)


 30     continue
c=============End: loop through cell interfaces==============================

c==========Capacity for mapping from latitude longitude to physical space====

        if (mcapa.gt.0) then
         do i=2-mbc,mx+mbc
          if (ixy.eq.1) then
            dxdc=(Rearth*pi/180.d0)
          else
	    dxdc=auxl(i,3)
          endif

          do mw=1,mwaves
	     s(i,mw)=dxdc*s(i,mw)
             do m=1,meqn
               fwave(i,m,mw)=dxdc*fwave(i,m,mw)
             enddo
          enddo
         enddo
        endif

c===============================================================================


c============= compute fluctuations=============================================
         do i=1-mbc,mx+mbc
            do m=1,meqn
               amdq(i,m)=0.0d0
               apdq(i,m)=0.0d0
               do  mw=1,mwaves
                  if (s(i,mw).lt.0.d0) then
                     amdq(i,m)=amdq(i,m) + fwave(i,m,mw)
                  elseif (s(i,mw).gt.0.d0) then
                     apdq(i,m)=apdq(i,m) + fwave(i,m,mw)
                  endif
               enddo
            enddo
         enddo


      return
      end


c=============================================================================
      subroutine riemann(hL,hR,uL,uR,hm,s1m,s2m,rare1,rare2,maxiter)
c=============================================================================

      implicit none

      integer iter,maxiter

      double precision abs_tol,grav,g,depthtolerance
      double precision hL,hR,uL,uR,hm,s1m,s2m,u1m,u2m,um,delu
      double precision h_max,h_min,h0,F_max,F_min,dfdh,F0,slope,gL,gR

      logical rare1,rare2

      common /comrp/ grav
      common /tolerance/ depthtolerance

      g=grav

c=====================================================================
c     Test for Riemann structure
c=====================================================================
         h_min=min(hR,hL)
         h_max=max(hR,hL)
         delu=uR-uL

	 if (h_min.le.0.d0) then
	     hm=0.d0
	     um=0.d0
	     s1m=uR+uL-2.d0*sqrt(g*hR)+2.d0*sqrt(g*hL)
	     s2m=uR+uL-2.d0*sqrt(g*hR)+2.d0*sqrt(g*hL)
	     if (hL.le.0.d0) then
		rare2=.true.
		rare1=.false.
	     else
		rare1=.true.
		rare2=.false.
	     endif
	 else

         F_min= delu+2.d0*(sqrt(g*h_min)-sqrt(g*h_max))
         F_max= delu +
     &         (h_max-h_min)*(sqrt(.5d0*g*(h_max+h_min)/(h_max*h_min)))

         if (F_min.gt.0.d0) then !2-rarefactions

             hm=(1.d0/(16.d0*g))*
     &               max(0.d0,-delu+2.d0*(sqrt(g*hL)+sqrt(g*hR)))**2
             um=sign(1.d0,hm)*(uL+2.d0*(sqrt(g*hL)-sqrt(g*hm)))

             s1m=uL+2.d0*sqrt(g*hL)-3.d0*sqrt(g*hm)
             s2m=uR-2.d0*sqrt(g*hR)+3.d0*sqrt(g*hm)

	     rare1=.true.
   	     rare2=.true.

         elseif (F_max.le.0.d0) then !2 shocks

c               ===root finding using a Newton iteration on sqrt(h)===
                h0=h_max
                do iter=1,maxiter
                    gL=sqrt(.5d0*g*(1/h0 + 1/hL))
                    gR=sqrt(.5d0*g*(1/h0 + 1/hR))
                    F0=delu+(h0-hL)*gL + (h0-hR)*gR
                    dfdh=gL-g*(h0-hL)/(4.d0*(h0**2)*gL)+
     &                   gR-g*(h0-hR)/(4.d0*(h0**2)*gR)
                    slope=2.d0*sqrt(h0)*dfdh
                    h0=(sqrt(h0)-F0/slope)**2
                enddo
c               ======================================================
                hm=h0
                u1m=uL-(hm-hL)*sqrt((.5d0*g)*(1/hm + 1/hL))
                u2m=uR+(hm-hR)*sqrt((.5d0*g)*(1/hm + 1/hR))
		um=.5d0*(u1m+u2m)

                s1m=u1m-sqrt(g*hm)
                s2m=u2m+sqrt(g*hm)

		rare1=.false.
		rare2=.false.

         else !one shock one rarefaction
                h0=h_min
c               ===== root find using secant iteration on sqrt(h)=====
                do iter=1,maxiter
                    F0=delu + 2.d0*(sqrt(g*h0)-sqrt(g*h_max))
     &                  + (h0-h_min)*sqrt(.5d0*g*(1/h0+1/h_min))
                    slope=(F_max-F0)/(sqrt(h_max)-sqrt(h0))
                    h0=(sqrt(h0)-F0/slope)**2
                enddo
c               =====================================================
                hm=h0
                if (hL.gt.hR) then
                    um=uL+2.d0*sqrt(g*hL)-2.d0*sqrt(g*hm)
                    s1m=uL+2.d0*sqrt(g*hL)-3.d0*sqrt(g*hm)
                    s2m=uL+2.d0*sqrt(g*hL)-sqrt(g*hm)
		    rare1=.true.
		    rare2=.false.
                else
                    s2m=uR-2.d0*sqrt(g*hR)+3.d0*sqrt(g*hm)
                    s1m=uR-2.d0*sqrt(g*hR)+sqrt(g*hm)
                    um=uR-2.d0*sqrt(g*hR)+2.d0*sqrt(g*hm)
		    rare2=.true.
		    rare1=.false.
                endif

         endif
	 endif


c=====================================================================
c   RIEMANN STRUCTURE KNOWN
c=====================================================================
      return

      end
